@@ -25,6 +25,7 @@ install("nltk")
 install("textblob")
 ##----------------------------------------------IMPORTS---------------------------------------------------------------------------#	
 import json
+import ast
 import pandas as pd
 import numpy as np
 import urllib3
@@ -61,7 +62,7 @@ def sentiental_analysis_features(data):
 
 def distribution_sentiment_tweets(data):
 	### VISUALIZATION
-	plt.rcParams['figure.figsize'] = [50, 40]
+	plt.rcParams['figure.figsize'] = [40, 20]
 	for i in range (len(data)):
 		x=data.polarity.iloc[i]
 		y=data.subjectivity.iloc[i]
@@ -71,8 +72,7 @@ def distribution_sentiment_tweets(data):
 			else: 
 				plt.scatter(x,y, color='pink')
 		elif -0.1<=x<=0.1:
-			if 0.4<=y<=0.6:  # yellow   green
-				plt.scatter(x,y, color='gold')
+			plt.scatter(x,y, color='gold')
 		else:
 			if y>0.6:  # yellow   green
 				plt.scatter(x,y, color='purple')
@@ -81,12 +81,13 @@ def distribution_sentiment_tweets(data):
 	plt.text(x+0.01, y+0.01, 'tweet'+str(i), fontsize=10)
 	plt.xlim(-1,1)
 	plt.ylim(0,1)
-	plt.title('sentiment anaysis',fontsize=20)
+	plt.title('Sentiment anaysis',fontsize=20)
 	plt.xlabel('<------------------ NEGATIVE ------------------   POLARITY ------------------ POSITIVE ------------------>',fontsize=15)
 	plt.ylabel('<----------------- FACTS ------------------ SUBJECTIVITY ------------------ OPINIONS ----------------->',fontsize=15)
 	plt.show()
 
 def general_pieChart(data):
+	plt.rcParams['figure.figsize'] = [9, 6]	
 	p=0 # positive
 	n=0 # Negative
 	o=0 # neutral
@@ -122,10 +123,31 @@ def general_pieChart(data):
 	# Plot
 	ax1.pie(sizes, explode=explode, labels=labels, colors=colors, autopct='%1.1f%%', shadow=True, startangle=140)
 	ax2.pie(sizes2, explode=explode2, labels=labels2, colors=colors2, autopct='%1.1f%%', shadow=True, startangle=140)
-	plt.title('The General Sentiment Analysis of all The tweets of ')
+	plt.title('The General Sentiment Analysis of all The tweets')
 	plt.axis('equal')
 	plt.show()
 	
+def final_pieChart(data):
+
+    plt.rcParams['figure.figsize'] = [9, 6]
+
+    po = data["positive_opinion"]
+    pf = data["positive_fact"]
+    neu = data["neutral"]
+    nf = data["negative_fact"]
+    no = data["negative_opinion"]
+    labels = 'Positive opinion', 'Positive fact','Neutral','Negative fact','Negative opinion'
+    sizes = [po,pf,neu,nf,no]
+    colors = ['blue','red', 'gold','yellowgreen', 'lightcoral','lightskyblue']
+    explode = (0,0,0,0,0)  # explode 1st slice
+
+    fig, (ax1) = plt.subplots(1, 1)
+    # Plot
+    ax1.pie(sizes, explode=explode, labels=labels, colors=colors, autopct='%1.1f%%', shadow=True, startangle=140)
+    plt.title('Sentiment of the tweets of the profile')
+    plt.axis('equal')
+    plt.show()
+	
 def tokenize_tweets(data):
 	sample_data_tokenized=[]  
 	tokenizer=RegexpTokenizer(r'\w+')
@@ -159,14 +181,15 @@ def keyword_score(data,sample_data_tokenized, keyword_corpus):
 	data['keyword_score']=scores
 	return data
 
-def hashtag_score(data, hastags_corpus):
+
+def hashtag_score(data, hashtags_corpus):
 	scores=[]
 	data = define_hashtags(data)
 	for hshtgs in data.hashtags:
 		score = 0
 		for h in hshtgs:
 			#print(h)
-			for H in hastags_corpus.hashtags:
+			for H in hashtags_corpus.hashtags:
 				if h == H:
 					score += 1
 		scores.append(score)
@@ -193,6 +216,7 @@ def terrorist_keywords_belonging(data):
 	plt.show()
 	
 def pie_chart_keywords_terrorist(data):
+	plt.rcParams['figure.figsize'] = [9, 6]
 	data_terr = data[data['keyword_score']>0].reset_index(drop=True)
 	p=0
 	n=0
@@ -232,8 +256,8 @@ def terrorist_hashtags_belonging(data):
 	temp = data[ data['hashtag_score']>0].reset_index(drop=True)
 	# Draw plot
 	fig, ax = plt.subplots(figsize=(36,24), dpi= 80)
-	ax.vlines(x=temp.index, ymin=0, ymax=temp.hashtags_score, color='firebrick', alpha=0.7, linewidth=2)
-	ax.scatter(x=temp.index, y=temp.hashtags_score, s=75, color='firebrick', alpha=0.7)
+	ax.vlines(x=temp.index, ymin=0, ymax=temp.hashtag_score, color='firebrick', alpha=0.7, linewidth=2)
+	ax.scatter(x=temp.index, y=temp.hashtag_score, s=75, color='firebrick', alpha=0.7)
 	# Title, Label, Ticks and Ylim
 	ax.set_title('Lollipop Chart for The belonging each tweet to the Terrorist Hashtag Network', fontdict={'size':22})
 	ax.set_ylabel('Number of Terrorist Hashtags' )
@@ -242,10 +266,11 @@ def terrorist_hashtags_belonging(data):
 	ax.set_ylim(0, 30)
 	# Annotate
 	for row in temp.itertuples():
-		ax.text(row.Index+0.01, row.hashtags_score+0.15,rotation=90, s=round(row.hashtags_score, 2), horizontalalignment= 'center', verticalalignment='bottom', fontsize=14)
+		ax.text(row.Index+0.01, row.hashtag_score+0.15,rotation=90, s=round(row.hashtag_score, 2), horizontalalignment= 'center', verticalalignment='bottom', fontsize=14)
 	plt.show()
 
 def pie_chart_hashtags_terrorist(data):
+	plt.rcParams['figure.figsize'] = [9, 6]
 	temp = data[data['hashtag_score']>0].reset_index(drop=True)  
 	p=0
 	n=0
@@ -279,7 +304,28 @@ def pie_chart_hashtags_terrorist(data):
 	ax2.pie(sizes2, explode=explode2, labels=labels2, colors=colors2, autopct='%1.1f%%', shadow=True, startangle=140)
 	plt.title('from '+str(len(data))+' tweets, '+ str(len(temp))+' tweets are following Terroristic Hashtags')
 	plt.axis('equal')
-	plt.show()  
+	plt.show() 
+
+def final_pieChart(data):
+
+    plt.rcParams['figure.figsize'] = [9, 6]
+
+    po = data["positive_opinion"]
+    pf = data["positive_fact"]
+    neu = data["neutral"]
+    nf = data["negative_fact"]
+    no = data["negative_opinion"]
+    labels = 'Positive opinion', 'Positive fact','Neutral','Negative fact','Negative opinion'
+    sizes = [po,pf,neu,nf,no]
+    colors = ['blue','red', 'gold','yellowgreen', 'lightcoral']
+    explode = (0,0,0,0,0)  # explode 1st slice
+
+    fig, (ax1) = plt.subplots(1, 1)
+    # Plot
+    ax1.pie(sizes, explode=explode, labels=labels, colors=colors, autopct='%1.1f%%', shadow=True, startangle=140)
+    plt.title('Sentiment of the tweets of the profile')
+    plt.axis('equal')
+    plt.show()	
 
 #----------------------------------------FUNCTIONS-----------------------------------------------------------------------------#
 
@@ -322,9 +368,10 @@ def profile_connections (data_tweets):
     #Agruping
     lst_col = "screen_name_mention"
     data = data[["screen_name","screen_name_mention"]]
+	#Create new rows in case of a tweet has different mentions
     data_ext =pd.DataFrame({col:np.repeat(data[col].values, data[lst_col].str.len())for col in data.columns.difference(
         [lst_col])}).assign(**{lst_col:np.concatenate(data[lst_col].values)})[data.columns.tolist()]
-    data1_ext = pd.DataFrame({'self_loop_iteration' : data_ext.groupby(data_ext.columns.tolist(),as_index=False).size().sort_values(
+    data1_ext = pd.DataFrame({'iteration' : data_ext.groupby(data_ext.columns.tolist(),as_index=False).size().sort_values(
                                                                         ascending = False)}).reset_index()
     return data1_ext
 
@@ -358,7 +405,7 @@ def self_loop_iteration_nodes(df):
     for i in range(len(df)):
         if(df["screen_name"].iloc[i] ==df["screen_name_mention"].iloc[i]):
             n.append(df['screen_name'].iloc[i])
-            self_loop_iteration.append(df["self_loop_iteration"].iloc[i])            
+            self_loop_iteration.append(df["iteration"].iloc[i])            
         else:
             n.append(df['screen_name'].iloc[i])
             n.append(df['screen_name_mention'].iloc[i])
@@ -454,164 +501,163 @@ def graph_features(DiG,df):
     output = merge_dataframes(output, df_nodes_self_loop_iteration)
     return output
 
-def user_timeline_extraction (node_list, consumerKey, consumerSecret,accessToken,accessTokenSecret, iter_tw, fichero):
-    # get_user Returns the 20 most recent statuses posted from the authenticating user or the user specified. 
-    profile_iter = len(node_list)
-    k=0
-    auth = tweepy.OAuthHandler(consumerKey[0], consumerSecret[0])
-    auth.set_access_token(accessToken[0], accessTokenSecret[0])
-    api = tweepy.API(auth)
-    nc= 0
-    ultima = 0
-    origen1 = time.time()
-    while ((k<= (len(node_list)+profile_iter)) & (ultima !=2) ):
-      tweets_user=[]
-      list_users_noInfo = []
-      origen2 = time.time()
-      i=0
-      if ((len(node_list)>= k) & ((len(node_list)-profile_iter) <=k)):
-        node_list1 = node_list[k:len(node_list)]
-        ultima = 1
-      else:
-        node_list1 = node_list[k:k+profile_iter]
-      while(i <(len(node_list1))):         
-        pri = 0
-        itera=0
-        max_id=""
-        intentos = 0
-        while itera != iter_tw:
-          try:
-            if pri ==0:
-              user=api.user_timeline(node_list1[i], count=200,tweet_mode="extended")      
-              tweets_user.append(user)
-              max_id = str(tweets_user[len(tweets_user)-1][len(tweets_user[len(tweets_user)-1])-1]._json["id_str"])
-              pri=1
-              itera = itera+1
-            else:
-              user=api.user_timeline(node_list1[i], count=200,tweet_mode="extended", max_id=max_id)
-              tweets_user.append(user)
-              max_id = str(tweets_user[len(tweets_user)-1][len(tweets_user[len(tweets_user)-1])-1]._json["id_str"])
-              itera = itera+1
-              intentos=0 # Para si justo es el primero el que no se puede extraer y no es problema de agotar claves
-          except:
-            nc = nc+1
-            n_clave = len(consumerKey)
-            if(nc == n_clave):
-              nc= 0
-            auth = tweepy.OAuthHandler(consumerKey[nc], consumerSecret[nc])
-            auth.set_access_token(accessToken[nc], accessTokenSecret[nc])
-            api = tweepy.API(auth)
-            intentos = intentos + 1
-            if(intentos == 2):         
-              list_users_noInfo.append(node_list1[i])
-              intentos = 0
-              itera=iter_tw
-        i = i+1       
-      destino2 = time.time()
-      aux=[]
-      for i in range( len(tweets_user)):
-        for j in range(len(tweets_user[i])):          
-            aux.append(tweets_user[i][j]._json)     
-      if(len(aux) != 0):  
-        df = pd.DataFrame(aux)
-        df = df.rename(columns = {"created_at": "created_at_tw"})
-        df= df.rename(columns = {"lang": "lang_tw"})
-        df = pd.concat([df.drop(['user'], axis=1), df['user'].apply(pd.Series)], axis=1)
-        df = df.rename(columns={"followers_count": "followers",
-                              "friends_count": "followees",
-                              "retweet_count": "retweets",
-                              "favorite_count": "favorites",
-                              "favourites_count": "favorites_count",
-                              "full_text": "text"})
-        df2 = metadata_extraction(df)
-        if path.exists("/content/"+ fichero):
-          df2.to_csv(fichero,mode="a", index=False, header=False)
-        else:
-          df2.to_csv(fichero,index=False)         
-        if len(list_users_noInfo) !=0:
-          nodes = pd.DataFrame()
-          nodes["nodos"] = list_users_noInfo
-          if path.exists("/content/node_without_"+fichero):
-            nodes.to_csv("node_without_"+ fichero, mode ="a",index=False, header=False)          
-          else:
-             nodes.to_csv("node_without_"+ fichero, index=False)
-        k = k+profile_iter        
-        if ultima ==1:          
-          ultima =2
-      else:
-        print("Repetimos")
-    print("Lista de usuarios sin metadata")
-    print(list_users_noInfo)
-    destino1 = time.time()
-    return df2, list_users_noInfo
+def user_timeline_extraction (node_list, consumerKey, consumerSecret,accessToken,accessTokenSecret, iter_tw,save_by, fichero):
+	profile_iter = save_by
+	k=0
+	auth = tweepy.OAuthHandler(consumerKey[0], consumerSecret[0])
+	auth.set_access_token(accessToken[0], accessTokenSecret[0])
+	api = tweepy.API(auth)
+	nc= 0
+	ultima = 0
+	origen1 = time.time()
+	while ((k<= (len(node_list)+profile_iter)) & (ultima !=2) ):
+		tweets_user=[]
+		list_users_noInfo = []
+		origen2 = time.time()
+		i=0
+		if ((len(node_list)>= k) & ((len(node_list)-profile_iter) <=k)):
+			node_list1 = node_list[k:len(node_list)]
+			ultima = 1
+		else:
+			node_list1 = node_list[k:k+profile_iter]
+		while(i <(len(node_list1))):         
+			pri = 0
+			itera=0
+			max_id=""
+			intentos = 0
+			while itera != iter_tw:
+			try:
+				if pri ==0:
+					user=api.user_timeline(node_list1[i], count=200,tweet_mode="extended")      
+					tweets_user.append(user)
+					max_id = str(tweets_user[len(tweets_user)-1][len(tweets_user[len(tweets_user)-1])-1]._json["id_str"])
+					pri=1
+					itera = itera+1
+				else:
+					user=api.user_timeline(node_list1[i], count=200,tweet_mode="extended", max_id=max_id)
+					tweets_user.append(user)
+					max_id = str(tweets_user[len(tweets_user)-1][len(tweets_user[len(tweets_user)-1])-1]._json["id_str"])
+					itera = itera+1
+					intentos=0 # Para si justo es el primero el que no se puede extraer y no es problema de agotar claves
+			except:
+				nc = nc+1
+				n_clave = len(consumerKey)
+				if(nc == n_clave):
+					nc= 0
+				auth = tweepy.OAuthHandler(consumerKey[nc], consumerSecret[nc])
+				auth.set_access_token(accessToken[nc], accessTokenSecret[nc])
+				api = tweepy.API(auth)
+				intentos = intentos + 1
+			if(intentos == 2):         
+				list_users_noInfo.append(node_list1[i])
+				intentos = 0
+				itera=iter_tw
+			i = i+1       
+		destino2 = time.time()
+		aux=[]
+		for i in range( len(tweets_user)):
+			for j in range(len(tweets_user[i])):          
+				aux.append(tweets_user[i][j]._json)     
+		if(len(aux) != 0):  
+			df = pd.DataFrame(aux)
+			df = df.rename(columns = {"created_at": "created_at_tw"})
+			df= df.rename(columns = {"lang": "lang_tw"})
+			df = pd.concat([df.drop(['user'], axis=1), df['user'].apply(pd.Series)], axis=1)
+			df = df.rename(columns={"followers_count": "followers",
+							  "friends_count": "followees",
+							  "retweet_count": "retweets",
+							  "favorite_count": "favorites",
+							  "favourites_count": "favorites_count",
+							  "full_text": "text"})
+			df2 = metadata_extraction(df)
+			if path.exists("/content/"+ fichero):
+				df2.to_csv(fichero,mode="a", index=False, header=False)
+			else:
+				df2.to_csv(fichero,index=False)         
+			if len(list_users_noInfo) !=0:
+				nodes = pd.DataFrame()
+				nodes["nodos"] = list_users_noInfo
+			if path.exists("/content/node_without_"+fichero):
+				nodes.to_csv("node_without_"+ fichero, mode ="a",index=False, header=False)          
+			else:
+				nodes.to_csv("node_without_"+ fichero, index=False)
+			k = k+profile_iter        
+			if ultima ==1:          
+				ultima =2
+		else:
+			print("Repetimos")
+	print("Lista de usuarios sin metadata")
+	print(list_users_noInfo)
+	destino1 = time.time()
+	return df2, list_users_noInfo
 
 
 #----------------------------------Extract metadata-----------------------------------------------------------#
 def metadata_extraction (df_users):
     
-    users = df_users["screen_name"].unique()
-    count_users=0
-    df = pd.DataFrame() 
-    for i in range(len(users)):
-        data_tweets = df_users[df_users["screen_name"]==users[i]].reset_index()
-        
-        data = pd.DataFrame()              
-        if(data_tweets.empty == False):       
-            count_users = count_users + 1
-            print("Usuarios: %f" % count_users)
-            [screen_name_mention_user,seasonalityDF,tweets_urlDF,mentionsDF,text_tweetsDF,
-             fav_twDF_list,ret_twDF_list,fav_twDF,ret_twDF,RT_tw,ntwPro_DF,minu_tw,tw_per_days_list,
-             text_tweets_days_list,tweet_date_days_list,duplicated,minu_tw_answer,
-             tw_day_list] = text_tweets_features_extraction(data_tweets)
-            data["screen_name_mention_user"] = screen_name_mention_user
-            data["screen_name"] = data_tweets["screen_name"].iloc[0]  
-            data["created_at"] = data_tweets["created_at"].iloc[0]
-            data["following"] = data_tweets["followees"].iloc[0]
-            data["followers"] = data_tweets["followers"].iloc[0]
-            data["statuses_count"] = data_tweets["statuses_count"].iloc[0]
-            data["default_profile"] = data_tweets["default_profile"].iloc[0]
-            data["default_profile_image"] = data_tweets["default_profile_image"].iloc[0]
-            data["biography_profile"] = data_tweets["description"].iloc[0]
-            data["listed_count"] = data_tweets["listed_count"].iloc[0]    
-            data["favourite_count"] = data_tweets["favorites_count"].iloc[0]
-            data["seasonality"] = seasonalityDF    
-            data["tweets_url"] = tweets_urlDF    
-            data["mentions"] = mentionsDF 
-            data["text_tweets_days_list"] = text_tweets_days_list
-            data["tweet_date_days_list"] = tweet_date_days_list   
-            data["text_tweets"] = text_tweetsDF
-            data["favorite_tweets_list"] = fav_twDF_list
-            data["retweet_tweets_list"] = ret_twDF_list
-            data["favorite_tweets_count"] = fav_twDF
-            data["retweet_tweets_count"] = ret_twDF
-            data["RT"] = RT_tw
-            data["num_ownTw"] = ntwPro_DF    
-            data["time_btw_tw"] = minu_tw
-            data["tw_day_list"] = tw_day_list
-            data["tw_per_day_list"] = tw_per_days_list
-            data["tw_duplicated"] = duplicated
-            data["minu_tw_answer"] = minu_tw_answer
-            data["profile_type"] = data_tweets["verified"].iloc[0]
-            data["screen_name_mention_user"] = data["screen_name_mention_user"].astype(str)
-            probably_fake = []
-            for j in range(len(data)):
-                if data["biography_profile"].any()==False:# Si se lee el dataframe
-                #if len(data["biography_profile"][j]) == 0:# Si se hace la extraccion antes de guardar los datos
-                    data["biography_profile"][j] = 1
-                else:
-                    data["biography_profile"][j] = 0
-                if data["following"].iloc[j] == 2001:
-                    probably_fake.append(1)
-                else:
-                    probably_fake.append(0)
-            data["probably_fake"] = probably_fake
-            df = df.append(data, ignore_index=True)
-    df =df.loc[df.astype(str).drop_duplicates().index].reset_index()
-    
-    df = time_twitter_account(df)
-    df = time_series_tw(df)
+	users = df_users["screen_name"].unique()
+	count_users=0
+	df = pd.DataFrame() 
+	for i in range(len(users)):
+		data_tweets = df_users[df_users["screen_name"]==users[i]].reset_index()
+		
+		data = pd.DataFrame()              
+		if(data_tweets.empty == False):       
+			count_users = count_users + 1
+			print("Usuarios: %f" % count_users)
+			[screen_name_mention_user,seasonalityDF,tweets_urlDF,mentionsDF,text_tweetsDF,
+			 fav_twDF_list,ret_twDF_list,fav_twDF,ret_twDF,RT_tw,ntwPro_DF,minu_tw,tw_per_days_list,
+			 text_tweets_days_list,tweet_date_days_list,duplicated,minu_tw_answer,
+			 tw_day_list] = text_tweets_features_extraction(data_tweets)
+			data["screen_name_mention_user"] = screen_name_mention_user
+			data["screen_name"] = data_tweets["screen_name"].iloc[0]  
+			data["created_at"] = data_tweets["created_at"].iloc[0]
+			data["following"] = data_tweets["followees"].iloc[0]
+			data["followers"] = data_tweets["followers"].iloc[0]
+			data["statuses_count"] = data_tweets["statuses_count"].iloc[0]
+			data["default_profile"] = data_tweets["default_profile"].iloc[0]
+			data["default_profile_image"] = data_tweets["default_profile_image"].iloc[0]
+			data["biography_profile"] = data_tweets["description"].iloc[0]
+			data["listed_count"] = data_tweets["listed_count"].iloc[0]    
+			data["favourite_count"] = data_tweets["favorites_count"].iloc[0]
+			data["seasonality"] = seasonalityDF    
+			data["tweets_url"] = tweets_urlDF    
+			data["mentions"] = mentionsDF 
+			data["text_tweets_days_list"] = text_tweets_days_list
+			data["tweet_date_days_list"] = tweet_date_days_list   
+			data["text_tweets"] = text_tweetsDF
+			data["favorite_tweets_list"] = fav_twDF_list
+			data["retweet_tweets_list"] = ret_twDF_list
+			data["favorite_tweets_count"] = fav_twDF
+			data["retweet_tweets_count"] = ret_twDF
+			data["RT"] = RT_tw
+			data["num_ownTw"] = ntwPro_DF    
+			data["time_btw_tw"] = minu_tw
+			data["tw_day_list"] = tw_day_list
+			data["tw_per_day_list"] = tw_per_days_list
+			data["tw_duplicated"] = duplicated
+			data["minu_tw_answer"] = minu_tw_answer
+			data["profile_type"] = data_tweets["verified"].iloc[0]
+			data["screen_name_mention_user"] = data["screen_name_mention_user"].astype(str)
+			probably_fake = []
+			for j in range(len(data)):
+				if data["biography_profile"].any()==False:# Si se lee el dataframe
+				#if len(data["biography_profile"][j]) == 0:# Si se hace la extraccion antes de guardar los datos
+					data["biography_profile"][j] = 1
+				else:
+					data["biography_profile"][j] = 0
+				if data["following"].iloc[j] == 2001:
+					probably_fake.append(1)
+				else:
+					probably_fake.append(0)
+			data["probably_fake"] = probably_fake
+			df = df.append(data, ignore_index=True)
+	df =df.loc[df.astype(str).drop_duplicates().index].reset_index()
 
-    return df
+	df = time_twitter_account(df)
+	df = time_series_tw(df)
+	df = separate_text_tweets_feature(df)
+	return df
 
 
 
@@ -833,7 +879,7 @@ def advanced_features(data):
     data["max_tw_day"] = 0
     data["index_max_day_tw"]=0
     for i in range(len(data)):
-        if (len(data.iloc[i]["screen_name"]) > 4):
+        if (len(str(data.iloc[i]["screen_name"])) > 4):
             if((data.iloc[i]["screen_name"][-1].isdigit()) & (data.iloc[i]["screen_name"][-2].isdigit()) & 
                 (data.iloc[i]["screen_name"][-3].isdigit()) &(data.iloc[i]["screen_name"][-4].isdigit())):
                 data["screen_name_bot"].iloc[i] = 1
@@ -881,9 +927,11 @@ def trend_tweets_features(data):
     for i in range(len(data)):
         #trend rates
         if len(data.iloc[i]["tw_per_day_list"]) >4:
+			#Select 95% confidence interval of a normal distribution to say that a values of tweet per day is an outlier
             df_aux= pd.DataFrame(data.iloc[i]["tw_per_day_list"])
-            df_aux["floor"] = df_aux[0].mean() - 1.33 * df_aux[0].std()
-            df_aux["roof"] = df_aux[0].mean() + 1.33 * df_aux[0].std() 
+            df_aux["floor"] = df_aux[0].mean() - 1.64 * df_aux[0].std()
+            df_aux["roof"] = df_aux[0].mean() + 1.64 * df_aux[0].std() 
+			#
             df_aux["anom"] = df_aux.apply(lambda row: row[0] if (row[0]<= row["floor"] or 
                                                                  row[0]>= row["roof"]) else 0, axis =1) 
             data["num_anom"].iloc[i] = len(list(filter(lambda x: x != 0, df_aux["anom"])))
@@ -903,6 +951,37 @@ def trend_tweets_features(data):
             data["num_anom"].iloc[i] =0
             data["anom_rate"].iloc[i]=0
     return data
+	
+def profile_trend_visualization(profile_evaluated):
+	num_profile =0
+	time_likes = pd.Series(data=profile_evaluated['tw_per_day_list'], index=profile_evaluated['tw_day_list'])
+	time_likes.plot(figsize=(20, 8), color='r')
+	plt.title('Trend of the tweets of the profile',fontsize=20)
+	plt.xlabel('Date',fontsize=15)
+	plt.ylabel('Number of tweets',fontsize=15)
+	plt.show()
+	
+def profile_max_tw_day_hourly_visualization(data):
+  print(type(data["tweet_date_days_list"]))
+  data["tweet_date_days_list_copy"] = copy.deepcopy(data["tweet_date_days_list"])
+  date_list =data['tweet_date_days_list_copy'][data["index_max_day_tw"]]
+  for i in range(len(date_list)):
+    date_list[i] = date_list[i].time().hour    
+  x = list(range(0,24))
+  y = []
+  for i in range(len(x)):
+    count=0
+    for j in range(len(date_list)):        
+      if date_list[j] == i:
+        count = count + 1
+    y.append(count)
+  del data["tweet_date_days_list_copy"]
+  plt.title('Trend of the tweets of the profile(max tweets day "hourly")',fontsize=20)
+  plt.xlabel('Date',fontsize=15)
+  plt.ylabel('Number of tweets',fontsize=15)
+  plt.axis([0,23,0, max(y)+5])
+  plt.locator_params(axis='x', nbins=24)
+  plt.plot(x,y)
 #----------------------------------------------Filters-------------------------------------------------------#
 
 
@@ -920,7 +999,7 @@ def filter_possible_irregular_profiles(data_join):
                   (data_join["max_tw_day"]<40) & (data_join["tw_RT_rate"]<0.2) &
                  ((data_join["max_ret_tw"]>500) | (data_join["max_fav_tw"]>500))] ="NLP-F1"
     
-    #Profiles with a high number of RT in a especific day (try to spread information)
+    #Profiles with a high number of RT in a specific day (try to spread information)
     data_join["evaluate_NLP"][(data_join["trend"] == 0) & (data_join["profile_type"]==False) &
                   (data_join["max_tw_day"]>150) & (data_join["tw_RT_rate"]<0.1) & 
                 (data_join["evaluate_NLP"]=="No")] = "NLP-F2"
@@ -949,7 +1028,7 @@ def filter_possible_irregular_profiles(data_join):
                   (data_join["max_tw_day"]>180) &  (data_join["evaluate_NLP"]=="No")] = "NLP-F7"
     
     #Profiles BOT (more than tweet or RT each 5 minutes the 24 hours)
-    data_join["evaluate_NLP"][(data_join["max_tw_day"]>288) &(data_join["profile_type"]==False) &
+    data_join["evaluate_NLP"][(data_join["max_tw_day"]>	288) &(data_join["profile_type"]==False) &
                              (data_join["evaluate_NLP"]=="No")] ="NLP-8"
     
     data_join["evaluate_NLP"][(data_join["trend"]==2) & (data_join["profile_type"]==False) & 
@@ -958,44 +1037,117 @@ def filter_possible_irregular_profiles(data_join):
     return data_join
 
 def NLP_analysis(data, keyword_corpus, hashtag_corpus):
-	tweets = data["text_tweets_days_list"]
-	tweets_join = [j for i in tweets for j in i]
-	tweets_df = pd.DataFrame(tweets_join)
-	tweets_df = tweets_df[tweets_df["RT"] !=2].reset_index(drop=True)
-	tweets_df = sentiental_analysis_features(tweets_df)
-	sample_data_tokenized = tokenize_tweets(tweets_df)
-	tweets_df = keyword_score(tweets_df, sample_data_tokenized,keyword_corpus)
-	tweets_df = hashtag_score(tweets_df,hashtag_corpus)	
-	tweets_df["terrorist_suggestion"] = 0
-	tweets_df["terrorist_suggestion"][(tweets_df["hashtag_score"]>0) | (tweets_df["keyword_score"]>0)] = 1
-	if len(tweets_df[tweets_df["terrorist_suggestion"]==1]) == 0:
-		data["per_hash_key"] = 0
-		data["per_terr_hash_key"] = 0
-	else:
-		data["per_hash_key"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==1]) / len(tweets_df))
-		tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < -0.1) & (tweets_df["subjectivity"]>0.6)]= 2
-		if len(tweets_df[tweets_df["terrorist_suggestion"]==2]) == 0:
-			data["per_terr_hash_key"] = 0
-		else:
-			print(tweets_df.groupby("terrorist_suggestion").size())
-			if len(tweets_df[tweets_df["terrorist_suggestion"]==1]) == 0:
-				data["per_terr_hash_key"] =1
-			else:
-				data["per_terr_hash_key"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==2]) / len(tweets_df[tweets_df["terrorist_suggestion"]==1]))
-	data["suggestion"] = "No terrorist"
-	if(data["per_terr_hash_key"] >0.7) & (data["per_hash_key"]>0.15):
-		data["suggestion"]="Terrorist"
-	return data,tweets_df
+    tweets = data["text_tweets_days_list"]
+    tweets_join = [j for i in tweets for j in i]
+    tweets_df = pd.DataFrame(tweets_join)
+    tweets_df = tweets_df[tweets_df["RT"] !=2].reset_index(drop=True)
+    tweets_df = sentiental_analysis_features(tweets_df)
+    sample_data_tokenized = tokenize_tweets(tweets_df)
+    tweets_df = keyword_score(tweets_df, sample_data_tokenized,keyword_corpus)
+    tweets_df = hashtag_score(tweets_df,hashtag_corpus)	
+    tweets_df["terrorist_suggestion"] = 0
+    print(data["screen_name"])
+    tweets_df["terrorist_suggestion"][(tweets_df["hashtag_score"]>0) | (tweets_df["keyword_score"]>0)] = 1
+    tweets_irr_words = len(tweets_df[tweets_df["terrorist_suggestion"]==1])
+    if len(tweets_df[tweets_df["terrorist_suggestion"]==1]) == 0:
+      data["negative_opinion"] = 0
+      data["negative_fact"] = 0
+      data["neutral"] = 0
+      data["positive_fact"] = 0
+      data["positive_opinion"] = 0
+    else:
+      data["per_hash_key"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==1]) / len(tweets_df))
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < -0.1) & (tweets_df["subjectivity"]>0.6)]= 2
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < -0.1) & (tweets_df["subjectivity"]<0.6)]= 3
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < 0.1) & (tweets_df["subjectivity"]>0.2) & (tweets_df["polarity"] > -0.1) & (tweets_df["subjectivity"]<0.4)]= 4
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] > 0.1) & (tweets_df["subjectivity"]<0.6)]= 5
+      tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] > 0.1) & (tweets_df["subjectivity"]>0.6)]= 6
+      if len(tweets_df[tweets_df["terrorist_suggestion"]==2]) == 0:
+        data["negative_opinion"] = 0
+      else:
+        data["negative_opinion"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==2]) / tweets_irr_words)
+      if len(tweets_df[tweets_df["terrorist_suggestion"]==3]) == 0:
+        data["negative_fact"] = 0
+      else:
+        data["negative_fact"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==3]) / tweets_irr_words)
+      if len(tweets_df[tweets_df["terrorist_suggestion"]==4]) == 0:
+        data["neutral"] = 0
+      else:
+        data["neutral"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==4]) / tweets_irr_words)
+      if len(tweets_df[tweets_df["terrorist_suggestion"]==5]) == 0:
+        data["positive_fact"] = 0
+      else:
+        data["positive_fact"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==5]) / tweets_irr_words)
+      if len(tweets_df[tweets_df["terrorist_suggestion"]==6]) == 0:
+        data["positive_opinion"] = 0
+      else:
+        data["positive_opinion"] = (len(tweets_df[tweets_df["terrorist_suggestion"]==6]) / tweets_irr_words)
+      
+    return data,tweets_df
 	
-def more_tweets_required(df,consumerKey,consumerSecret,accessToken,accessTokenSecret):
+def more_tweets_required(df,df_graph_features,consumerKey,consumerSecret,accessToken,accessTokenSecret,n_groups):
 	df_evaluate = df[(df["evaluate_NLP"] !="No") & (df["evaluate_NLP"] !="More Data")]
 	node_list_moreData = list(df[df["evaluate_NLP"]=="More Data"]["screen_name"])
 	if len(node_list_moreData) >0:
-		df_metadata_moreData, list_more_Data_noData = user_timeline_extraction(node_list_moreData,consumerKey,consumerSecret,accessToken,accessTokenSecret,5,"fichero_guardar_noData.csv")
+		df_metadata_moreData, list_more_Data_noData = user_timeline_extraction(node_list_moreData,consumerKey,consumerSecret,accessToken,accessTokenSecret,n_groups,save_by,"fichero_guardar_noData.csv")
+		df_metadata_moreData = pd.read_csv("/content/"+"fichero_guardar_noData.csv")
+		df_metadata_moreData = text_to_list_features_conversor(df_metadata_moreData)
+		df_metadata_moreData = advanced_features(df_metadata_moreData)
+		df_metadata_moreData = merge_dataframes(df_metadata_moreData,df_graph_features)
 		df_metadata_moreData = filter_possible_irregular_profiles(df_metadata_moreData)
 		df_metadata_moreData = df_metadata_moreData[(df_metadata_moreData["evaluate_NLP"] !="No") & (df_metadata_moreData["evaluate_NLP"] !="More Data")]
 		df_evaluate = df_evaluate.append(df_metadata_moreData, ignore_index=True)
 	return df_evaluate
+	
+def text_to_list_features_conversor (data):
+    for index, row in data.iterrows():
+        print("Nuevo usuario")
+        print (index)
+        data["favorite_tweets_list"].iloc[index] = ast.literal_eval(data.loc[index, "favorite_tweets_list"])
+        data["retweet_tweets_list"].iloc[index] = ast.literal_eval(data.loc[index,"retweet_tweets_list"])
+        data["tw_per_day_list"].iloc[index] = ast.literal_eval(data.loc[index,"tw_per_day_list"])
+        data["text_tweets_days_list"].iloc[index] = data.loc[index,"text_tweets_days_list"].replace('"', "'").replace("], [","?#??").strip("[]'").split("?#??")
+        list_day = []                                                                                
+        list_text_tweets_day = []
+        list_tweet_date_days = []
+        list_RT_days = []
+        list_lang_days = []
+        text_days = data.loc[index,"tw_day_list"].strip("][").replace(")", ")??").split("??, ")
+        tweet_date_days = data.loc[index,"tweet_date_days_list"].replace("], [","???").split("???")
+        RT_days = data.loc[index, "RT_days_list"].strip("[]").split("], [")
+        lang_days = data.loc[index, "lang_days_list"].strip("[]").split("], [")
+        
+        for i in range(len(text_days)):
+            #print(i)
+            tweet_date_day = tweet_date_days[i].replace("), ",")??").strip("[]").split("??")
+            text_tweets_day = data["text_tweets_days_list"].iloc[index][i].replace("'}, {'","'}??{'").strip("[]").split("??")
+            if len(RT_days[i])==1:
+                 RT_day = list(RT_days[i])
+            else:
+                RT_day = list(ast.literal_eval(str(RT_days[i])))
+            lang_day = list(ast.literal_eval(lang_days[i]))
+            list_tweet_date_day = []
+            for j in range(len(tweet_date_day)):
+                
+                if(tweet_date_day == ["'Empty'"]):
+                    list_tweet_date_day.append("Empty")
+                else:
+                    list_tweet_date_day.append(datetime.strptime(tweet_date_day[j],"Timestamp('%Y-%m-%d %H:%M:%S%z', tz='UTC')"))    
+                
+            if i == len(text_days)-1:
+                text_days[i] = text_days[i].replace("??","")
+            list_day.append(datetime.strptime(text_days[i],'datetime.date(%Y, %m, %d)' ).date())
+            list_text_tweets_day.append(text_tweets_day)
+            list_tweet_date_days.append(list_tweet_date_day)
+            list_RT_days.append(RT_day)
+            list_lang_days.append(lang_day)
+        data["text_tweets_days_list"].iloc[index] = list_text_tweets_day
+        data["tw_day_list"].iloc[index]= list_day
+        data["tweet_date_days_list"].iloc[index]= list_tweet_date_days
+        data["RT_days_list"].iloc[index] = list_RT_days
+        data["lang_days_list"].iloc[index] = list_lang_days
+    return data
+
 
 	
 def evaluate_NLP_profiles(data,keyword_corpus, hashtag_corpus):	
@@ -1003,4 +1155,45 @@ def evaluate_NLP_profiles(data,keyword_corpus, hashtag_corpus):
 	for i in range(len(data)):
 		profile_evaluated, tweets_df = NLP_analysis(data.iloc[i], keyword_corpus, hashtag_corpus)
 		final_df = final_df.append(profile_evaluated, ignore_index=True)
-	return final_df
\ No newline at end of file
+	return final_df
+	
+def separate_text_tweets_feature(df_metadata):
+  df_metadata["RT_days_list"] = 0
+  df_metadata["lang_days_list"] = 0
+  for k in range(len(df_metadata)):
+    data_tweets = df_metadata.iloc[k]["text_tweets_days_list"]
+    text_tweets_day = []
+    text_tweets_days = []
+    lang_text_tweets_day = []
+    lang_text_tweets_days = []
+    RT_text_tweets_day = []
+    RT_text_tweets_days = []
+    for i in range(len(data_tweets)):
+      text_tweets_day = []
+      lang_text_tweets_day = []
+      RT_text_tweets_day = []
+      if (len(data_tweets) ==1) & (len(data_tweets[0])==3):
+        try:
+          text_tweets_days.append(data_tweets[i]["text"])
+          lang_text_tweets_days.append(data_tweets[i]["lang"])
+          RT_text_tweets_days.append(str(data_tweets[i]["RT"]))
+        except:
+          for j in range(len(data_tweets[i])):
+            text_tweets_day.append(data_tweets[i][j]["text"])
+            lang_text_tweets_day.append(data_tweets[i][j]["lang"])
+            RT_text_tweets_day.append(str(data_tweets[i][j]["RT"]))
+          text_tweets_days.append(text_tweets_day)
+          lang_text_tweets_days.append(lang_text_tweets_day)
+          RT_text_tweets_days.append(RT_text_tweets_day) 
+      else:
+        for j in range(len(data_tweets[i])):
+          text_tweets_day.append(data_tweets[i][j]["text"])
+          lang_text_tweets_day.append(data_tweets[i][j]["lang"])
+          RT_text_tweets_day.append(str(data_tweets[i][j]["RT"]))
+        text_tweets_days.append(text_tweets_day)
+        lang_text_tweets_days.append(lang_text_tweets_day)
+        RT_text_tweets_days.append(RT_text_tweets_day)
+    df_metadata["text_tweets_days_list"].iloc[k] = text_tweets_days    
+    df_metadata["lang_days_list"].iloc[k] = lang_text_tweets_days
+    df_metadata["RT_days_list"].iloc[k]= RT_text_tweets_days
+  return df_metadata
\ No newline at end of file
